// ScreenEffects v1.0 shader file

extern float brightness <
    string UIName = "Brightness";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 4.0f;
    float UIStep = 0.01f;
> = 1;

extern float shadows <
    string UIName = "Shadows";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 4.0f;
    float UIStep = 0.01f;
> = 1;

extern float highlights <
    string UIName = "Highlights";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 4.0f;
    float UIStep = 0.01f;
> = 1;

extern float contrast <
    string UIName = "Contrast";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 4.0f;
    float UIStep = 0.01f;
> = 1;

extern float saturation <
    string UIName = "Saturation";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 4.0f;
    float UIStep = 0.01f;
> = 1;

extern int p_steps <
    string UIName = "Posterize Steps";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 20.0f;
    float UIStep = 1.0f;
> = 0;

extern float blurdist <
    string UIName = "Blur Distance";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 0.01f;
    float UIStep = 0.0005f;
> = 0.00;

extern float gfoffset <
    string UIName = "Graduated Filter Offset";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 1.0f;
    float UIStep = 0.01f;
> = 0;

extern float gfmagnitude <
    string UIName = "Graduated Filter Magnitude";
    string UIWidget = "slider";
    float UIMin = 0.0f;
    float UIMax = 1.0f;
    float UIStep = 0.01f;
> = 0;


texture thisframe;	// the current frame from before any effects are applied from this file
texture lastpass;	// texture map from the last pass (used in between effect application)


sampler currentFrameSampler = sampler_state
{
	texture = <thisframe>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MINFILTER = POINT;
	MAGFILTER = POINT;
};

sampler lastPassSampler = sampler_state
{
	texture = <lastpass>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
};

struct VSOUT
{
	float4 vertPos : POSITION;
	float2 UVCoord : TEXCOORD0;
};

struct VSIN
{
	float4 vertPos : POSITION0;
	float2 UVCoord : TEXCOORD0;
};

VSOUT FrameVS(VSIN IN)
{
	VSOUT OUT = (VSOUT)0.0f;	// initialize to zero, avoid complaints.
	OUT.vertPos = IN.vertPos;
	OUT.UVCoord = IN.UVCoord;
	return OUT;
}


// Effect Functions
float4 doContrast (float4 color, float amt)
{
		color -= 0.5;
		color *= amt;
		color += 0.5;
    return color;
}

float4 doShadows (float4 color, float amt)
{
    half lum = (color.r + color.g + color.b)/3;
    if (lum < 1./3.) {
	    	half4 ncolor = color - 0.5;
	    	ncolor *= amt;
	    	ncolor += 0.5;
        ncolor = lerp(ncolor,color,lum * 3);
        return ncolor;
    }
    else {
        return color;
    }
}

float4 doHighlights (float4 color, float amt)
{
    half lum = (color.r + color.g + color.b)/3;
    if (lum > .5) {
	    	half4 ncolor = color - 0.5;
	    	ncolor *= amt;
	    	ncolor += 0.5;
        ncolor = lerp(color,ncolor,lum * 2 - 1);
        return ncolor;
    }
    else {
        return color;
    }
}

float4 doBrightness (float4 color, float amt)
{
		return color * amt;
}

float4 doSaturation (float4 color, float amt)
{
		float grey_s = ((color.r * 0.3) + (color.g * 0.59)) + (color.b * 0.11);
		return(lerp(grey_s, color, amt));
}

float4 doGradFilter (float4 color, float2 UVCoord, float gfo, float gfm)
{
    float y1 = 1. - UVCoord.y;
    if (y1 < gfo)
    {
        return color;
    }
    else
    {
        return lerp(color, color * (1. - (y1 - gfo) / (1. - gfo)), gfm);
    }
}

float4 doPosterize (float4 color,	int steps)
{
    if (steps > 0)
    {
		    color = (color * steps);
		    color = (floor(color));
		    color /= steps;
        return color;
    }
    else
    {
        return color;
    }
}

float3 doDiskBlur (float amt, float2 UVCoord)
{
    // poisson disk blur
    const float3 pSamples[] ={   {0.000000,0.000000,1},
                                 {-0.929317,-0.295797,1},
                                 {0.525823,0.672132,1},
                                 {0.867314,0.132608,1},
                                 {0.002469,-0.692395,1},
                                 {-0.273576,0.914886,1},
                                 {-0.729743,0.451020,1},
                                 {0.544711,-0.537214,1},
                                 {-0.607088,-0.736707,1}, };
                                         
   float3 retCol = 0;
   float weights = 0;
   for (int i=0;  i < 9;  i++ )
   {
       retCol += tex2D(lastPassSampler, UVCoord + (float2)pSamples[i].xy * blurdist);
       weights += (float)pSamples[i].z;
   }
   return retCol/weights;
}

// Main
half4 ApplyScreenEffects(float2 UVCoord : TEXCOORD0) : COLOR0
{
    half4 baseColor = tex2D(currentFrameSampler, UVCoord);
    half3 newColor = baseColor.rgb;
  
    newColor = doBrightness(float4(newColor,1),brightness);
    newColor = doShadows(float4(newColor,1),shadows);
    newColor = doHighlights(float4(newColor,1),highlights);    
    newColor = doContrast(float4(newColor,1),contrast);
    newColor = doSaturation(float4(newColor,1),saturation);    
    newColor = doGradFilter(float4(newColor,1),UVCoord, gfoffset, gfmagnitude);
    newColor = doPosterize(float4(newColor,1),p_steps);    
   
    return half4(newColor, 1);
}

half4 ApplyBlur(VSOUT IN) : COLOR0
{
    half3 newColor = doDiskBlur(blurdist,IN.UVCoord);
    return half4(newColor, 1);
}


technique t0
{

	pass p0
	{
;		//VertexShader = compile vs_1_1 FrameVS();	// unneeded unless the compile target is PS3.0
		PixelShader = compile ps_2_0 ApplyScreenEffects();
		CullMode = none;
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = false;
	}
    
  pass p1
	{
;		//VertexShader = compile vs_1_1 FrameVS();	// unneeded unless the compile target is PS3.0
		PixelShader = compile ps_2_0 ApplyBlur();
		CullMode = none;
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = false;
	}
    
	// Additional passes can be commented out to improve shader performance
  pass p2
	{
;		//VertexShader = compile vs_1_1 FrameVS();	// unneeded unless the compile target is PS3.0
		PixelShader = compile ps_2_0 ApplyBlur();
		CullMode = none;
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = false;
	}
  
  pass p3
	{
;		//VertexShader = compile vs_1_1 FrameVS();	// unneeded unless the compile target is PS3.0
		PixelShader = compile ps_2_0 ApplyBlur();
		CullMode = none;
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = false;
	}
    
}